// This file is made available under Elastic License 2.0.
// This file is based on code available under the Apache license here:
//   https://github.com/apache/incubator-doris/blob/master/fe/fe-core/src/test/java/org/apache/doris/common/FeNameFormatTest.java

// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package com.starrocks.common;

import org.junit.Test;

import java.util.Random;

public class FeNameFormatTest {

    @Test
    public void testCheckColumnName() {

        ExceptionChecker.expectThrowsNoException(() -> FeNameFormat.checkColumnName("_id"));
        ExceptionChecker.expectThrowsNoException(() -> FeNameFormat.checkColumnName("01"));
        ExceptionChecker.expectThrowsNoException(() -> FeNameFormat.checkColumnName("Space Test"));
        ExceptionChecker.expectThrowsNoException(() -> FeNameFormat.checkColumnName("@timestamp"));

        String rndStr = "0123456789qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM";
        Random r = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 1; i <= 64; i++) {
            sb.append(rndStr.charAt(r.nextInt(rndStr.length())));
        }
        // length 64
        ExceptionChecker.expectThrowsNoException(() -> FeNameFormat.checkColumnName(sb.toString()));
        ExceptionChecker.expectThrows(AnalysisException.class, () -> FeNameFormat.checkColumnName("Column \0 Name"));
        ExceptionChecker.expectThrows(AnalysisException.class, () -> FeNameFormat.checkColumnName("\0"));
        // length 0
        ExceptionChecker.expectThrows(AnalysisException.class, () -> FeNameFormat.checkColumnName(""));

    }

}
